/*
  -----------------Note----------------------
  Date: 5-29-2020
  Project Name: MrK_Blockvader
  Goal: Basic operate system
  Author: Khang Nguyen           
*/

#include <SPI.h>
#include <nRF24L01.h>
#include <RF24.h>

#define RMotor 5
#define LMotor 6

#define RMotor_In1  4
#define RMotor_In2  2

#define LMotor_In1  7
#define LMotor_In2  8

#define eyes_LED 3

#define buzzerPin A0
#define Sensor_LED A1
#define Dis_Pin  A2
#define Photo_Pin  A3

RF24 radio(9, 10); // CE, CSN

const byte address[6] = "00001";


unsigned long lastReceiveTime = 0;
unsigned long currentTime = 0;

struct Data_Package 
{
  byte L_XJoy_Value;
  byte L_YJoy_Value;
  byte R_YJoy_Value;
  byte Throtle_Value;

  byte R_SJoy_State ;
  byte S_Switch_State;
  byte M_Switch_State;

  byte C_mode;
  
};

Data_Package data;

int singing = 0 ;
int Trim_Value = 0; 

int Arm_HPos = 88;
int Arm_Pos = 0 ;

int F_LED_State = LOW;
long F_LED_OnTime  = 0; 
long F_LED_OffTime = 0;
unsigned long p_F_LED_Millis = 0;  


int lightLevel;
int Current_Light;
int Last_Light ; 
int Ok_Light     = 0;
int Good_Light   = 0;


int Dis ; 
int last_Dis; 
int Ok_Dis; 
int Good_Dis;
int Bad_Dis;

void setup()
{
  Serial.begin(9600);
  setup_WorkerBot();
  firstSection();
  lightLevel = analogRead(Photo_Pin);
  Current_Light = lightLevel   ; 
}

void loop()
 {
  lightLevel = analogRead(Photo_Pin);
  Serial.print("Light level:  ");Serial.println(lightLevel);
  Serial.print("Current light:  ");Serial.println(Current_Light);
  if (lightLevel >= 200){ Chill_mode();analogWrite(eyes_LED, 50);Serial.println("Chill mode");}
  if (lightLevel < 180){ Active_mode();analogWrite(eyes_LED, 150);Serial.println("Active mode");}
 }
//==============Worker_Bot_Setup==================//
int setup_WorkerBot()
{
  pinMode(RMotor, OUTPUT);
  pinMode(RMotor_In1, OUTPUT);
  pinMode(RMotor_In2, OUTPUT);

  pinMode(LMotor, OUTPUT);
  pinMode(LMotor_In1, OUTPUT);
  pinMode(LMotor_In2, OUTPUT);

  
  pinMode(eyes_LED, OUTPUT);
  pinMode(Sensor_LED, OUTPUT);
  pinMode(buzzerPin, OUTPUT); 
  
}
//====================Data_Package====================//
void resetData()
{
  data.L_XJoy_Value = 127;
  data.L_YJoy_Value = 127;
  data.R_YJoy_Value = 127;
  data.Throtle_Value = 127;

  data.R_SJoy_State = 1 ;
  data.S_Switch_State = 1;
  data.M_Switch_State = 1;
}

//==============Mode=================//
void Direct_drive()
{
     
        Serial.print("Rover Mode: ");Serial.println(data.C_mode);
        
        if (data.Throtle_Value < 10){Trim_Value = 0;}
        
        if (data.L_YJoy_Value >117){move_Backward(data.Throtle_Value);}
        if (data.L_YJoy_Value <113){move_Forward(data.Throtle_Value);}
        
        if (data.L_YJoy_Value == 113){move_Forward(0);}

        if(data.L_XJoy_Value <103){turn_Left(data.Throtle_Value);}
        if(data.L_XJoy_Value >103){turn_Right(data.Throtle_Value);}

        if (data.M_Switch_State == 1){digitalWrite(eyes_LED, HIGH);}
        if (data.M_Switch_State == 0){digitalWrite(eyes_LED, LOW);}

        if (data.R_SJoy_State == 1 && singing ==0){firstSection();}
        if (data.R_SJoy_State == 0){singing = 0;}
}
void Autonomous_mode()
{
  int sensitivity = 10 ; 
  lightLevel = analogRead(Photo_Pin);
  Serial.print("Light level:  ");Serial.println(lightLevel);
 // Serial.print("Current light:  ");Serial.println(Current_Light);
  Serial.print("Rover Mode: ");Serial.println(data.C_mode);
  if( data.C_mode ==1)
  {
   if (lightLevel >= Current_Light ){ Chill_mode();Serial.println("Chill mode");}
  }
    if( data.C_mode ==1)
  {
  if (lightLevel < Current_Light ){ Active_mode();Serial.println("Active mode");}
  }
}

void Active_mode()
{
    if (lightLevel < 200)// dark
  {
   F_LED_OnTime  = 500; 
   F_LED_OffTime = 500;
  }

  int brightness = map(lightLevel , 0 , 600, 200, 20 );
  if (lightLevel > 600){brightness = 0; }

  lightLevel = analogRead(Photo_Pin);
  Ok_Light     = lightLevel + 50 ;
  Good_Light   = lightLevel + 100 ;
  
  move_SRover();
  }

void Chill_mode()
{
   move_Forward(0);
  if (lightLevel >= 500) // outdoor
  {
   F_LED_OnTime  = 0; 
   F_LED_OffTime = 1000;
  }
    if (lightLevel < 100) //room lighting
  {
   F_LED_OnTime  = 1500; 
   F_LED_OffTime = 1500;
  }
    if (lightLevel < 5)// dark
  {
   F_LED_OnTime  = 2500; 
   F_LED_OffTime = 0;
  }
  unsigned long c_Millis = millis();
  if((F_LED_State == HIGH) && (c_Millis - p_F_LED_Millis >= F_LED_OnTime))
  {
    F_LED_State = LOW;  
    p_F_LED_Millis = c_Millis; 
    digitalWrite(buzzerPin, F_LED_State); 
  }
  else if ((F_LED_State == LOW) && (c_Millis - p_F_LED_Millis >= F_LED_OffTime))
  {
    F_LED_State = HIGH;  
    p_F_LED_Millis = c_Millis;  
    digitalWrite(buzzerPin, F_LED_State);  
  }
      int ran_num = random(1,80000);
    if( ran_num != 10 ||ran_num != 40 ||ran_num != 80||ran_num != 100){move_Forward(0);}
    if(ran_num == 10){turn_Right(20+random(10,100));delay(random(50,250));}
    if(ran_num == 40){turn_Left(20+random(10,100));delay(random(50,250));}
    if(ran_num == 80){move_Forward(10+random(10,100));delay(random(50,250));}
    if(ran_num == 100){move_Backward(10+random(10,100));delay(random(50,250));}
 }

//==============Movement=================//

void move_SRover()
{
  if (lightLevel >= 200 || Dis > 500)
  {
    move_Forward(0);
    Serial.println("Stay");
  }
    
  if (lightLevel < Good_Light ) 
  {
    int ran_num = random(1,8);
    if(ran_num == 1 ||ran_num == 3 && Dis <100){turn_Right(100);delay(150);}
    if(ran_num == 2 ||ran_num == 4 && Dis <100){turn_Left(100);delay(150);}
    if(ran_num == 5 ||ran_num == 6||ran_num == 7 && Dis <100){move_Forward(100);}
    if(ran_num == 8 ){firstSection();}
    Serial.println("Maybe move");
  }
  
  if (lightLevel < Ok_Light&& Dis <100) 
   {
    move_Forward(80);
    Serial.println("Move Forward");
   }

}
void move_Forward(int SSpeed)
  {
   analogWrite(RMotor, SSpeed);
   analogWrite(LMotor, SSpeed);
    
   digitalWrite(RMotor_In1,HIGH);
   digitalWrite(RMotor_In2,LOW);
   
   digitalWrite(LMotor_In1,HIGH);
   digitalWrite(LMotor_In2,LOW);
  }

void move_Backward(int SSpeed)
  {
   analogWrite(RMotor, SSpeed);
   analogWrite(LMotor, SSpeed);
    
   digitalWrite(RMotor_In1,LOW);
   digitalWrite(RMotor_In2,HIGH);
   
   digitalWrite(LMotor_In1,LOW);
   digitalWrite(LMotor_In2,HIGH);
  }

void turn_Left(int SSpeed)
  {
   analogWrite(RMotor, SSpeed);
   analogWrite(LMotor, SSpeed);
    
   digitalWrite(RMotor_In1,HIGH);
   digitalWrite(RMotor_In2,LOW);
   
   digitalWrite(LMotor_In1,LOW);
   digitalWrite(LMotor_In2,HIGH);
  }

void turn_Right(int SSpeed)
  {
   analogWrite(RMotor, SSpeed);
   analogWrite(LMotor, SSpeed);
    
   digitalWrite(RMotor_In1,LOW);
   digitalWrite(RMotor_In2,HIGH);
   
   digitalWrite(LMotor_In1,HIGH);
   digitalWrite(LMotor_In2,LOW);
  }

//==============Buzzer_Mode=================//
void beep( int note, int duration)
{
  tone(buzzerPin, note, (duration)); //Play tone on buzzerPin
  digitalWrite(eyes_LED, 80);
  delay(duration);
  noTone(buzzerPin);//Stop tone on buzzerPin
  digitalWrite(eyes_LED, 0);
  delay(50);
}

void WB_thinking()
{
 for(int i=0;i<20;i++)
  {
  int BSpeed=random(0,200);
  beep(880,BSpeed);
  }
  Current_Light = analogRead(Photo_Pin);
  if(Current_Light<=50){digitalWrite(Sensor_LED, HIGH);}
  Serial.print("Current light:  ");Serial.println(Current_Light);
}

void firstSection()
{
const int c = 261;const int d = 294;const int e = 329;const int f = 349;
const int g = 391;const int gS = 415;const int a = 440;const int aS = 455;
const int b = 466;const int cH = 523;const int cSH = 554;const int dH = 587;
const int dSH = 622;const int eH = 659;const int fH = 698;
const int fSH = 740;const int gH = 784;const int gSH = 830;const int aH = 880;

  beep(a, 500); beep(a, 500);beep(a, 500);beep(f, 350);
  beep(cH, 150);beep(a, 500);beep(f, 350);beep(cH, 150);
  beep(a, 650);delay(250);
  beep(eH, 500); beep(eH, 500); beep(eH, 500); beep(fH, 350);
  beep(cH, 150);beep(gS, 500);beep(f, 350);beep(cH, 150);
  beep(a, 650);delay(250);
  singing = 1;
}
